# This file is auto-generated by scripts/mysetting_bindings.py

from enum import Enum
from typing import Union

from ._rbox import PySetting as _PySetting


class OnAirDisplay(str, Enum):
    Off = "Off"
    On = "On"


class LCDBrightness(str, Enum):
    One = "One"
    Two = "Two"
    Three = "Three"
    Four = "Four"
    Five = "Five"


class Quantize(str, Enum):
    Off = "Off"
    On = "On"


class AutoCueLevel(str, Enum):
    Minus78dB = "Minus78dB"
    Minus72dB = "Minus72dB"
    Minus66dB = "Minus66dB"
    Minus60dB = "Minus60dB"
    Minus54dB = "Minus54dB"
    Minus48dB = "Minus48dB"
    Minus42dB = "Minus42dB"
    Minus36dB = "Minus36dB"
    Memory = "Memory"


class Language(str, Enum):
    English = "English"
    French = "French"
    German = "German"
    Italian = "Italian"
    Dutch = "Dutch"
    Spanish = "Spanish"
    Russian = "Russian"
    Korean = "Korean"
    ChineseSimplified = "ChineseSimplified"
    ChineseTraditional = "ChineseTraditional"
    Japanese = "Japanese"
    Portuguese = "Portuguese"
    Swedish = "Swedish"
    Czech = "Czech"
    Hungarian = "Hungarian"
    Danish = "Danish"
    Greek = "Greek"
    Turkish = "Turkish"


class JogRingBrightness(str, Enum):
    Off = "Off"
    Dark = "Dark"
    Bright = "Bright"


class JogRingIndicator(str, Enum):
    Off = "Off"
    On = "On"


class SlipFlashing(str, Enum):
    Off = "Off"
    On = "On"


class DiscSlotIllumination(str, Enum):
    Off = "Off"
    Dark = "Dark"
    Bright = "Bright"


class EjectLock(str, Enum):
    Unlock = "Unlock"
    Lock = "Lock"


class Sync(str, Enum):
    Off = "Off"
    On = "On"


class PlayMode(str, Enum):
    Continue = "Continue"
    Single = "Single"


class QuantizeBeatValue(str, Enum):
    EighthBeat = "EighthBeat"
    QuarterBeat = "QuarterBeat"
    HalfBeat = "HalfBeat"
    FullBeat = "FullBeat"


class HotCueAutoLoad(str, Enum):
    Off = "Off"
    RekordboxSetting = "RekordboxSetting"
    On = "On"


class HotCueColor(str, Enum):
    Off = "Off"
    On = "On"


class NeedleLock(str, Enum):
    Unlock = "Unlock"
    Lock = "Lock"


class TimeMode(str, Enum):
    Elapsed = "Elapsed"
    Remain = "Remain"


class JogMode(str, Enum):
    Vinyl = "Vinyl"
    CDJ = "CDJ"


class AutoCue(str, Enum):
    Off = "Off"
    On = "On"


class MasterTempo(str, Enum):
    Off = "Off"
    On = "On"


class TempoRange(str, Enum):
    SixPercent = "SixPercent"
    TenPercent = "TenPercent"
    SixteenPercent = "SixteenPercent"
    Wide = "Wide"


class PhaseMeter(str, Enum):
    Type1 = "Type1"
    Type2 = "Type2"


class VinylSpeedAdjust(str, Enum):
    TouchRelease = "TouchRelease"
    Touch = "Touch"
    Release = "Release"


class JogDisplayMode(str, Enum):
    Auto = "Auto"
    Info = "Info"
    Simple = "Simple"
    Artwork = "Artwork"


class PadButtonBrightness(str, Enum):
    One = "One"
    Two = "Two"
    Three = "Three"
    Four = "Four"


class JogLCDBrightness(str, Enum):
    One = "One"
    Two = "Two"
    Three = "Three"
    Four = "Four"
    Five = "Five"


class WaveformDivisions(str, Enum):
    TimeScale = "TimeScale"
    Phrase = "Phrase"


class Waveform(str, Enum):
    Waveform = "Waveform"
    PhaseMeter = "PhaseMeter"


class BeatJumpBeatValue(str, Enum):
    HalfBeat = "HalfBeat"
    OneBeat = "OneBeat"
    TwoBeat = "TwoBeat"
    FourBeat = "FourBeat"
    EightBeat = "EightBeat"
    SixteenBeat = "SixteenBeat"
    ThirtytwoBeat = "ThirtytwoBeat"
    SixtyfourBeat = "SixtyfourBeat"


class ChannelFaderCurve(str, Enum):
    SteepTop = "SteepTop"
    Linear = "Linear"
    SteepBottom = "SteepBottom"


class CrossfaderCurve(str, Enum):
    ConstantPower = "ConstantPower"
    SlowCut = "SlowCut"
    FastCut = "FastCut"


class HeadphonesPreEQ(str, Enum):
    PostEQ = "PostEQ"
    PreEQ = "PreEQ"


class HeadphonesMonoSplit(str, Enum):
    MonoSplit = "MonoSplit"
    Stereo = "Stereo"


class BeatFXQuantize(str, Enum):
    Off = "Off"
    On = "On"


class MicLowCut(str, Enum):
    Off = "Off"
    On = "On"


class TalkOverMode(str, Enum):
    Advanced = "Advanced"
    Normal = "Normal"


class TalkOverLevel(str, Enum):
    Minus24dB = "Minus24dB"
    Minus18dB = "Minus18dB"
    Minus12dB = "Minus12dB"
    Minus6dB = "Minus6dB"


class MidiChannel(str, Enum):
    One = "One"
    Two = "Two"
    Three = "Three"
    Four = "Four"
    Five = "Five"
    Six = "Six"
    Seven = "Seven"
    Eight = "Eight"
    Nine = "Nine"
    Ten = "Ten"
    Eleven = "Eleven"
    Twelve = "Twelve"
    Thirteen = "Thirteen"
    Fourteen = "Fourteen"
    Fifteen = "Fifteen"
    Sixteen = "Sixteen"


class MidiButtonType(str, Enum):
    Toggle = "Toggle"
    Trigger = "Trigger"


class MixerDisplayBrightness(str, Enum):
    White = "White"
    One = "One"
    Two = "Two"
    Three = "Three"
    Four = "Four"
    Five = "Five"


class MixerIndicatorBrightness(str, Enum):
    One = "One"
    Two = "Two"
    Three = "Three"


class ChannelFaderCurveLongFader(str, Enum):
    Exponential = "Exponential"
    Smooth = "Smooth"
    Linear = "Linear"


class OverviewWaveformType(str, Enum):
    HalfWaveform = "HalfWaveform"
    FullWaveform = "FullWaveform"


class WaveformColor(str, Enum):
    Blue = "Blue"
    Rgb = "Rgb"
    TriBand = "TriBand"


class KeyDisplayFormat(str, Enum):
    Classic = "Classic"
    Alphanumeric = "Alphanumeric"


class WaveformCurrentPosition(str, Enum):
    Left = "Left"
    Center = "Center"


class Setting:
    def __init__(self, path: str = None):
        if path is None:
            self._setting = None
        else:
            self._setting = _PySetting(path)

    @classmethod
    def new_mysetting(cls, path: str) -> "Setting":
        """Create a new MySetting instance."""
        self = cls()
        self._setting = _PySetting.new_mysetting(path)
        return self

    @classmethod
    def new_mysetting2(cls, path: str) -> "Setting":
        self = cls()
        self._setting = _PySetting.new_mysetting2(path)
        return self

    @classmethod
    def new_djmmysetting(cls, path: str) -> "Setting":
        self = cls()
        self._setting = _PySetting.new_djmmysetting(path)
        return self

    @classmethod
    def new_devsetting(cls, path: str) -> "Setting":
        self = cls()
        self._setting = _PySetting.new_devsetting(path)
        return self

    @classmethod
    def load(cls, path: str) -> "Setting":
        self = cls()
        self._setting = _PySetting.load(path)
        return self

    def dump_copy(self, path: str) -> None:
        return self._setting.dump_copy(path)

    def dump(self) -> None:
        return self._setting.dump()

    def get_on_air_display(self) -> OnAirDisplay:
        return OnAirDisplay(self._setting.get_on_air_display())

    def set_on_air_display(self, value: Union[str, OnAirDisplay]) -> None:
        if isinstance(value, str):
            value = OnAirDisplay(value)
        self._setting.set_on_air_display(value.value)

    def get_lcd_brightness(self) -> LCDBrightness:
        return LCDBrightness(self._setting.get_lcd_brightness())

    def set_lcd_brightness(self, value: Union[str, LCDBrightness]) -> None:
        if isinstance(value, str):
            value = LCDBrightness(value)
        self._setting.set_lcd_brightness(value.value)

    def get_quantize(self) -> Quantize:
        return Quantize(self._setting.get_quantize())

    def set_quantize(self, value: Union[str, Quantize]) -> None:
        if isinstance(value, str):
            value = Quantize(value)
        self._setting.set_quantize(value.value)

    def get_auto_cue_level(self) -> AutoCueLevel:
        return AutoCueLevel(self._setting.get_auto_cue_level())

    def set_auto_cue_level(self, value: Union[str, AutoCueLevel]) -> None:
        if isinstance(value, str):
            value = AutoCueLevel(value)
        self._setting.set_auto_cue_level(value.value)

    def get_language(self) -> Language:
        return Language(self._setting.get_language())

    def set_language(self, value: Union[str, Language]) -> None:
        if isinstance(value, str):
            value = Language(value)
        self._setting.set_language(value.value)

    def get_jog_ring_brightness(self) -> JogRingBrightness:
        return JogRingBrightness(self._setting.get_jog_ring_brightness())

    def set_jog_ring_brightness(self, value: Union[str, JogRingBrightness]) -> None:
        if isinstance(value, str):
            value = JogRingBrightness(value)
        self._setting.set_jog_ring_brightness(value.value)

    def get_jog_ring_indicator(self) -> JogRingIndicator:
        return JogRingIndicator(self._setting.get_jog_ring_indicator())

    def set_jog_ring_indicator(self, value: Union[str, JogRingIndicator]) -> None:
        if isinstance(value, str):
            value = JogRingIndicator(value)
        self._setting.set_jog_ring_indicator(value.value)

    def get_slip_flashing(self) -> SlipFlashing:
        return SlipFlashing(self._setting.get_slip_flashing())

    def set_slip_flashing(self, value: Union[str, SlipFlashing]) -> None:
        if isinstance(value, str):
            value = SlipFlashing(value)
        self._setting.set_slip_flashing(value.value)

    def get_disc_slot_illumination(self) -> DiscSlotIllumination:
        return DiscSlotIllumination(self._setting.get_disc_slot_illumination())

    def set_disc_slot_illumination(self, value: Union[str, DiscSlotIllumination]) -> None:
        if isinstance(value, str):
            value = DiscSlotIllumination(value)
        self._setting.set_disc_slot_illumination(value.value)

    def get_eject_lock(self) -> EjectLock:
        return EjectLock(self._setting.get_eject_lock())

    def set_eject_lock(self, value: Union[str, EjectLock]) -> None:
        if isinstance(value, str):
            value = EjectLock(value)
        self._setting.set_eject_lock(value.value)

    def get_sync(self) -> Sync:
        return Sync(self._setting.get_sync())

    def set_sync(self, value: Union[str, Sync]) -> None:
        if isinstance(value, str):
            value = Sync(value)
        self._setting.set_sync(value.value)

    def get_play_mode(self) -> PlayMode:
        return PlayMode(self._setting.get_play_mode())

    def set_play_mode(self, value: Union[str, PlayMode]) -> None:
        if isinstance(value, str):
            value = PlayMode(value)
        self._setting.set_play_mode(value.value)

    def get_quantize_beat_value(self) -> QuantizeBeatValue:
        return QuantizeBeatValue(self._setting.get_quantize_beat_value())

    def set_quantize_beat_value(self, value: Union[str, QuantizeBeatValue]) -> None:
        if isinstance(value, str):
            value = QuantizeBeatValue(value)
        self._setting.set_quantize_beat_value(value.value)

    def get_hotcue_autoload(self) -> HotCueAutoLoad:
        return HotCueAutoLoad(self._setting.get_hotcue_autoload())

    def set_hotcue_autoload(self, value: Union[str, HotCueAutoLoad]) -> None:
        if isinstance(value, str):
            value = HotCueAutoLoad(value)
        self._setting.set_hotcue_autoload(value.value)

    def get_hotcue_color(self) -> HotCueColor:
        return HotCueColor(self._setting.get_hotcue_color())

    def set_hotcue_color(self, value: Union[str, HotCueColor]) -> None:
        if isinstance(value, str):
            value = HotCueColor(value)
        self._setting.set_hotcue_color(value.value)

    def get_needle_lock(self) -> NeedleLock:
        return NeedleLock(self._setting.get_needle_lock())

    def set_needle_lock(self, value: Union[str, NeedleLock]) -> None:
        if isinstance(value, str):
            value = NeedleLock(value)
        self._setting.set_needle_lock(value.value)

    def get_time_mode(self) -> TimeMode:
        return TimeMode(self._setting.get_time_mode())

    def set_time_mode(self, value: Union[str, TimeMode]) -> None:
        if isinstance(value, str):
            value = TimeMode(value)
        self._setting.set_time_mode(value.value)

    def get_jog_mode(self) -> JogMode:
        return JogMode(self._setting.get_jog_mode())

    def set_jog_mode(self, value: Union[str, JogMode]) -> None:
        if isinstance(value, str):
            value = JogMode(value)
        self._setting.set_jog_mode(value.value)

    def get_auto_cue(self) -> AutoCue:
        return AutoCue(self._setting.get_auto_cue())

    def set_auto_cue(self, value: Union[str, AutoCue]) -> None:
        if isinstance(value, str):
            value = AutoCue(value)
        self._setting.set_auto_cue(value.value)

    def get_master_tempo(self) -> MasterTempo:
        return MasterTempo(self._setting.get_master_tempo())

    def set_master_tempo(self, value: Union[str, MasterTempo]) -> None:
        if isinstance(value, str):
            value = MasterTempo(value)
        self._setting.set_master_tempo(value.value)

    def get_tempo_range(self) -> TempoRange:
        return TempoRange(self._setting.get_tempo_range())

    def set_tempo_range(self, value: Union[str, TempoRange]) -> None:
        if isinstance(value, str):
            value = TempoRange(value)
        self._setting.set_tempo_range(value.value)

    def get_phase_meter(self) -> PhaseMeter:
        return PhaseMeter(self._setting.get_phase_meter())

    def set_phase_meter(self, value: Union[str, PhaseMeter]) -> None:
        if isinstance(value, str):
            value = PhaseMeter(value)
        self._setting.set_phase_meter(value.value)

    def get_vinyl_speed_adjust(self) -> VinylSpeedAdjust:
        return VinylSpeedAdjust(self._setting.get_vinyl_speed_adjust())

    def set_vinyl_speed_adjust(self, value: Union[str, VinylSpeedAdjust]) -> None:
        if isinstance(value, str):
            value = VinylSpeedAdjust(value)
        self._setting.set_vinyl_speed_adjust(value.value)

    def get_jog_display_mode(self) -> JogDisplayMode:
        return JogDisplayMode(self._setting.get_jog_display_mode())

    def set_jog_display_mode(self, value: Union[str, JogDisplayMode]) -> None:
        if isinstance(value, str):
            value = JogDisplayMode(value)
        self._setting.set_jog_display_mode(value.value)

    def get_pad_button_brightness(self) -> PadButtonBrightness:
        return PadButtonBrightness(self._setting.get_pad_button_brightness())

    def set_pad_button_brightness(self, value: Union[str, PadButtonBrightness]) -> None:
        if isinstance(value, str):
            value = PadButtonBrightness(value)
        self._setting.set_pad_button_brightness(value.value)

    def get_jog_lcd_brightness(self) -> JogLCDBrightness:
        return JogLCDBrightness(self._setting.get_jog_lcd_brightness())

    def set_jog_lcd_brightness(self, value: Union[str, JogLCDBrightness]) -> None:
        if isinstance(value, str):
            value = JogLCDBrightness(value)
        self._setting.set_jog_lcd_brightness(value.value)

    def get_waveform_divisions(self) -> WaveformDivisions:
        return WaveformDivisions(self._setting.get_waveform_divisions())

    def set_waveform_divisions(self, value: Union[str, WaveformDivisions]) -> None:
        if isinstance(value, str):
            value = WaveformDivisions(value)
        self._setting.set_waveform_divisions(value.value)

    def get_waveform(self) -> Waveform:
        return Waveform(self._setting.get_waveform())

    def set_waveform(self, value: Union[str, Waveform]) -> None:
        if isinstance(value, str):
            value = Waveform(value)
        self._setting.set_waveform(value.value)

    def get_beat_jump_beat_value(self) -> BeatJumpBeatValue:
        return BeatJumpBeatValue(self._setting.get_beat_jump_beat_value())

    def set_beat_jump_beat_value(self, value: Union[str, BeatJumpBeatValue]) -> None:
        if isinstance(value, str):
            value = BeatJumpBeatValue(value)
        self._setting.set_beat_jump_beat_value(value.value)

    def get_channel_fader_curve(self) -> ChannelFaderCurve:
        return ChannelFaderCurve(self._setting.get_channel_fader_curve())

    def set_channel_fader_curve(self, value: Union[str, ChannelFaderCurve]) -> None:
        if isinstance(value, str):
            value = ChannelFaderCurve(value)
        self._setting.set_channel_fader_curve(value.value)

    def get_crossfader_curve(self) -> CrossfaderCurve:
        return CrossfaderCurve(self._setting.get_crossfader_curve())

    def set_crossfader_curve(self, value: Union[str, CrossfaderCurve]) -> None:
        if isinstance(value, str):
            value = CrossfaderCurve(value)
        self._setting.set_crossfader_curve(value.value)

    def get_headphones_pre_eq(self) -> HeadphonesPreEQ:
        return HeadphonesPreEQ(self._setting.get_headphones_pre_eq())

    def set_headphones_pre_eq(self, value: Union[str, HeadphonesPreEQ]) -> None:
        if isinstance(value, str):
            value = HeadphonesPreEQ(value)
        self._setting.set_headphones_pre_eq(value.value)

    def get_headphones_mono_split(self) -> HeadphonesMonoSplit:
        return HeadphonesMonoSplit(self._setting.get_headphones_mono_split())

    def set_headphones_mono_split(self, value: Union[str, HeadphonesMonoSplit]) -> None:
        if isinstance(value, str):
            value = HeadphonesMonoSplit(value)
        self._setting.set_headphones_mono_split(value.value)

    def get_beat_fx_quantize(self) -> BeatFXQuantize:
        return BeatFXQuantize(self._setting.get_beat_fx_quantize())

    def set_beat_fx_quantize(self, value: Union[str, BeatFXQuantize]) -> None:
        if isinstance(value, str):
            value = BeatFXQuantize(value)
        self._setting.set_beat_fx_quantize(value.value)

    def get_mic_low_cut(self) -> MicLowCut:
        return MicLowCut(self._setting.get_mic_low_cut())

    def set_mic_low_cut(self, value: Union[str, MicLowCut]) -> None:
        if isinstance(value, str):
            value = MicLowCut(value)
        self._setting.set_mic_low_cut(value.value)

    def get_talk_over_mode(self) -> TalkOverMode:
        return TalkOverMode(self._setting.get_talk_over_mode())

    def set_talk_over_mode(self, value: Union[str, TalkOverMode]) -> None:
        if isinstance(value, str):
            value = TalkOverMode(value)
        self._setting.set_talk_over_mode(value.value)

    def get_talk_over_level(self) -> TalkOverLevel:
        return TalkOverLevel(self._setting.get_talk_over_level())

    def set_talk_over_level(self, value: Union[str, TalkOverLevel]) -> None:
        if isinstance(value, str):
            value = TalkOverLevel(value)
        self._setting.set_talk_over_level(value.value)

    def get_midi_channel(self) -> MidiChannel:
        return MidiChannel(self._setting.get_midi_channel())

    def set_midi_channel(self, value: Union[str, MidiChannel]) -> None:
        if isinstance(value, str):
            value = MidiChannel(value)
        self._setting.set_midi_channel(value.value)

    def get_midi_button_type(self) -> MidiButtonType:
        return MidiButtonType(self._setting.get_midi_button_type())

    def set_midi_button_type(self, value: Union[str, MidiButtonType]) -> None:
        if isinstance(value, str):
            value = MidiButtonType(value)
        self._setting.set_midi_button_type(value.value)

    def get_mixer_display_brightness(self) -> MixerDisplayBrightness:
        return MixerDisplayBrightness(self._setting.get_mixer_display_brightness())

    def set_mixer_display_brightness(self, value: Union[str, MixerDisplayBrightness]) -> None:
        if isinstance(value, str):
            value = MixerDisplayBrightness(value)
        self._setting.set_mixer_display_brightness(value.value)

    def get_mixer_indicator_brightness(self) -> MixerIndicatorBrightness:
        return MixerIndicatorBrightness(self._setting.get_mixer_indicator_brightness())

    def set_mixer_indicator_brightness(self, value: Union[str, MixerIndicatorBrightness]) -> None:
        if isinstance(value, str):
            value = MixerIndicatorBrightness(value)
        self._setting.set_mixer_indicator_brightness(value.value)

    def get_channel_fader_curve_long_fader(self) -> ChannelFaderCurveLongFader:
        return ChannelFaderCurveLongFader(self._setting.get_channel_fader_curve_long_fader())

    def set_channel_fader_curve_long_fader(
        self, value: Union[str, ChannelFaderCurveLongFader]
    ) -> None:
        if isinstance(value, str):
            value = ChannelFaderCurveLongFader(value)
        self._setting.set_channel_fader_curve_long_fader(value.value)

    def get_overview_waveform_type(self) -> OverviewWaveformType:
        return OverviewWaveformType(self._setting.get_overview_waveform_type())

    def set_overview_waveform_type(self, value: Union[str, OverviewWaveformType]) -> None:
        if isinstance(value, str):
            value = OverviewWaveformType(value)
        self._setting.set_overview_waveform_type(value.value)

    def get_waveform_color(self) -> WaveformColor:
        return WaveformColor(self._setting.get_waveform_color())

    def set_waveform_color(self, value: Union[str, WaveformColor]) -> None:
        if isinstance(value, str):
            value = WaveformColor(value)
        self._setting.set_waveform_color(value.value)

    def get_key_display_format(self) -> KeyDisplayFormat:
        return KeyDisplayFormat(self._setting.get_key_display_format())

    def set_key_display_format(self, value: Union[str, KeyDisplayFormat]) -> None:
        if isinstance(value, str):
            value = KeyDisplayFormat(value)
        self._setting.set_key_display_format(value.value)

    def get_waveform_current_position(self) -> WaveformCurrentPosition:
        return WaveformCurrentPosition(self._setting.get_waveform_current_position())

    def set_waveform_current_position(self, value: Union[str, WaveformCurrentPosition]) -> None:
        if isinstance(value, str):
            value = WaveformCurrentPosition(value)
        self._setting.set_waveform_current_position(value.value)



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-06-01

from pathlib import Path
from typing import Dict, List, Optional, Union

from . import models
from ._rbox import PyMasterDb as _PyMasterDb
from .anlz import Anlz


class MasterDb:
    def __init__(self, path: Union[str, Path] = None) -> None:
        if path is None:
            db = _PyMasterDb.open()
        else:
            path = Path(path)
            if not path.exists():
                raise FileNotFoundError(f"File not found: {path}")
            db = _PyMasterDb(str(path))
        self._db = db

    @classmethod
    def open(cls) -> "MasterDb":
        return cls()

    def set_unsafe_writes(self, unsafe: bool) -> None:
        """Set unsafe writes mode."""
        self._db.set_unsafe_writes(unsafe)

    # -- AgentRegistry ----------------------------------------------------------------------------

    def get_agent_registry(self) -> List[models.AgentRegistry]:
        """Get all agent registry entries."""
        return self._db.get_agent_registry()

    def get_agent_registry_by_id(self, registry_id: str) -> Optional[models.AgentRegistry]:
        """Get agent registry by ID."""
        return self._db.get_agent_registry_by_id(registry_id)

    def get_local_usn(self) -> int:
        """Get local USN."""
        return self._db.get_local_usn()

    # -- CloudAgentRegistry -----------------------------------------------------------------------

    def get_cloud_agent_registry(self) -> List[models.CloudAgentRegistry]:
        """Get all cloud agent registry entries."""
        return self._db.get_cloud_agent_registry()

    def get_cloud_agent_registry_by_id(
        self, registry_id: str
    ) -> Optional[models.CloudAgentRegistry]:
        """Get cloud agent registry by ID."""
        return self._db.get_cloud_agent_registry_by_id(registry_id)

    # -- ContentActiveCensor ----------------------------------------------------------------------

    def get_content_active_censor(self) -> List[models.ContentActiveCensor]:
        """Get all content active censor entries."""
        return self._db.get_content_active_censor()

    def get_content_active_censor_by_id(
        self, censor_id: str
    ) -> Optional[models.ContentActiveCensor]:
        """Get content active censor by ID."""
        return self._db.get_content_active_censor_by_id(censor_id)

    # -- ContentCue -------------------------------------------------------------------------------

    def get_content_cue(self) -> List[models.ContentCue]:
        """Get all content cue entries."""
        return self._db.get_content_cue()

    def get_content_cue_by_id(self, cue_id: str) -> Optional[models.ContentCue]:
        """Get content cue by ID."""
        return self._db.get_content_cue_by_id(cue_id)

    # -- ContentFile ------------------------------------------------------------------------------

    def get_content_file(self) -> List[models.ContentFile]:
        """Get all content file entries."""
        return self._db.get_content_file()

    def get_content_file_by_id(self, file_id: str) -> Optional[models.ContentFile]:
        """Get content file by ID."""
        return self._db.get_content_file_by_id(file_id)

    # -- ActiveCensor -----------------------------------------------------------------------------

    def get_active_censor(self) -> List[models.DjmdActiveCensor]:
        """Get all active censor entries."""
        return self._db.get_active_censor()

    def get_active_censor_by_id(self, censor_id: str) -> Optional[models.DjmdActiveCensor]:
        """Get active censor by ID."""
        return self._db.get_active_censor_by_id(censor_id)

    # -- Album ------------------------------------------------------------------------------------

    def get_album(self) -> List[models.DjmdAlbum]:
        """Get all album entries."""
        return self._db.get_album()

    def get_album_by_id(self, album_id: str) -> Optional[models.DjmdAlbum]:
        """Get album by ID."""
        return self._db.get_album_by_id(album_id)

    def get_album_by_name(self, album_name: str) -> Optional[models.DjmdAlbum]:
        """Get album by name."""
        return self._db.get_album_by_name(album_name)

    def insert_album(
        self, name: str, artist_id: str = None, image_path: str = None, compilation: int = None
    ) -> models.DjmdAlbum:
        """Insert a new album."""
        return self._db.insert_album(name, artist_id, image_path, compilation)

    def update_album(self, album: models.DjmdAlbum) -> models.DjmdAlbum:
        """Update album."""
        return self._db.update_album(album)

    def delete_album(self, album_id: str) -> None:
        """Delete album."""
        self._db.delete_album(album_id)

    # -- Artist -----------------------------------------------------------------------------------

    def get_artist(self) -> List[models.DjmdArtist]:
        """Get all artist entries."""
        return self._db.get_artist()

    def get_artist_by_id(self, artist_id: str) -> Optional[models.DjmdArtist]:
        """Get artist by ID."""
        return self._db.get_artist_by_id(artist_id)

    def get_artist_by_name(self, artist_name: str) -> Optional[models.DjmdArtist]:
        """Get artist by name."""
        return self._db.get_artist_by_name(artist_name)

    def insert_artist(self, name: str) -> models.DjmdArtist:
        """Insert a new artist."""
        return self._db.insert_artist(name)

    def update_artist(self, artist: models.DjmdArtist) -> models.DjmdArtist:
        """Update artist."""
        return self._db.update_artist(artist)

    def delete_artist(self, artist_id: str) -> None:
        """Delete artist."""
        self._db.delete_artist(artist_id)

    # -- Category ---------------------------------------------------------------------------------

    def get_category(self) -> List[models.DjmdCategory]:
        """Get all category entries."""
        return self._db.get_category()

    def get_category_by_id(self, category_id: str) -> Optional[models.DjmdCategory]:
        """Get category by ID."""
        return self._db.get_category_by_id(category_id)

    # -- Color ------------------------------------------------------------------------------------

    def get_color(self) -> List[models.DjmdColor]:
        """Get all color entries."""
        return self._db.get_color()

    def get_color_by_id(self, color_id: str) -> Optional[models.DjmdColor]:
        """Get color by ID."""
        return self._db.get_color_by_id(color_id)

    # -- Content ----------------------------------------------------------------------------------

    def get_content(self) -> List[models.DjmdContent]:
        """Get all content from the database."""
        return self._db.get_content()

    def get_content_by_id(self, content_id: str) -> Optional[models.DjmdContent]:
        """Get content by ID."""
        return self._db.get_content_by_id(content_id)

    def get_content_by_path(self, path: str) -> Optional[models.DjmdContent]:
        """Get content by path."""
        return self._db.get_content_by_path(path)

    def get_content_anlz_dir(self, content_id: str) -> Path:
        """Get the ANLZ directory for a content ID."""
        return Path(self._db.get_content_anlz_dir(content_id))

    def get_content_anlz_paths(self, content_id: str) -> Dict[str, Path]:
        """Get the ANLZ paths for a content ID."""
        paths = self._db.get_content_anlz_paths(content_id)
        return {ext: Path(p) for ext, p in paths.items()}

    def get_content_anlz_files(self, content_id: str) -> Dict[str, Anlz]:
        """Get the ANLZ file handlers for a content ID."""
        paths = self._db.get_content_anlz_paths(content_id)
        return {ext: Anlz(p) for ext, p in paths.items()}

    def insert_content(self, path: Union[Path, str]) -> models.DjmdContent:
        """Insert a new content into the database."""
        return self._db.insert_content(str(path))

    def update_content(self, content: models.DjmdContent) -> None:
        """Update content in the database."""
        self._db.update_content(content)

    def update_content_album(self, content_id: str, name: str) -> None:
        self._db.update_content_album(content_id, name)

    def update_content_artist(self, content_id: str, name: str) -> None:
        self._db.update_content_artist(content_id, name)

    def update_content_remixer(self, content_id: str, name: str) -> None:
        self._db.update_content_remixer(content_id, name)

    def update_content_original_artist(self, content_id: str, name: str) -> None:
        self._db.update_content_original_artist(content_id, name)

    def update_content_composer(self, content_id: str, name: str) -> None:
        self._db.update_content_composer(content_id, name)

    def update_content_genre(self, content_id: str, name: str) -> None:
        self._db.update_content_genre(content_id, name)

    def update_content_label(self, content_id: str, name: str) -> None:
        self._db.update_content_label(content_id, name)

    def update_content_key(self, content_id: str, name: str) -> None:
        self._db.update_content_key(content_id, name)

    # def delete_content(self, content_id: str) -> None:
    #     """Delete content from the database."""
    #     self._db.delete_content(content_id)

    # -- Cue --------------------------------------------------------------------------------------

    def get_cue(self) -> List[models.DjmdCue]:
        """Get all cue entries."""
        return self._db.get_cue()

    def get_cue_by_id(self, cue_id: str) -> Optional[models.DjmdCue]:
        """Get cue by ID."""
        return self._db.get_cue_by_id(cue_id)

    # -- Device -----------------------------------------------------------------------------------

    def get_device(self) -> List[models.DjmdDevice]:
        """Get all device entries."""
        return self._db.get_device()

    def get_device_by_id(self, device_id: str) -> Optional[models.DjmdDevice]:
        """Get device by ID."""
        return self._db.get_device_by_id(device_id)

    # -- Genre ------------------------------------------------------------------------------------

    def get_genre(self) -> List[models.DjmdGenre]:
        """Get all genre entries."""
        return self._db.get_genre()

    def get_genre_by_id(self, genre_id: str) -> Optional[models.DjmdGenre]:
        """Get genre by ID."""
        return self._db.get_genre_by_id(genre_id)

    def get_genre_by_name(self, genre_name: str) -> Optional[models.DjmdGenre]:
        """Get genre by ID."""
        return self._db.get_genre_by_name(genre_name)

    def insert_genre(self, name: str) -> models.DjmdGenre:
        """Insert a new genre."""
        return self._db.insert_genre(name)

    def update_genre(self, genre: models.DjmdGenre) -> models.DjmdGenre:
        """Update genre."""
        return self._db.update_genre(genre)

    def delete_genre(self, genre_id: str) -> None:
        """Delete genre."""
        self._db.delete_genre(genre_id)

    # -- History ----------------------------------------------------------------------------------

    def get_history(self) -> List[models.DjmdHistory]:
        """Get all history entries."""
        return self._db.get_history()

    def get_history_by_id(self, history_id: str) -> Optional[models.DjmdHistory]:
        """Get history by ID."""
        return self._db.get_history_by_id(history_id)

    def get_history_songs(self, history_id: str) -> List[models.DjmdSongHistory]:
        """Get songs in a history by ID."""
        return self._db.get_history_songs(history_id)

    def get_history_contents(self, history_id: str) -> List[models.DjmdContent]:
        """Get contents in a history by ID."""
        return self._db.get_history_contents(history_id)

    # -- HotCueBanklist ---------------------------------------------------------------------------

    def get_hot_cue_banklist(self) -> List[models.DjmdHotCueBanklist]:
        """Get all hot cue banklist entries."""
        return self._db.get_hot_cue_banklist()

    def get_hot_cue_banklist_by_id(self, banklist_id: str) -> Optional[models.DjmdHotCueBanklist]:
        """Get hot cue banklist by ID."""
        return self._db.get_hot_cue_banklist_by_id(banklist_id)

    def get_hot_cue_banklist_children(self, banklist_id: str) -> List[models.DjmdHotCueBanklist]:
        """Get child hot cue banklists by ID."""
        return self._db.get_hot_cue_banklist_children(banklist_id)

    def get_hot_cue_banklist_songs(self, banklist_id: str) -> List[models.DjmdSongHotCueBanklist]:
        """Get songs in a hot cue banklist by ID."""
        return self._db.get_hot_cue_banklist_songs(banklist_id)

    def get_hot_cue_banklist_contents(self, banklist_id: str) -> List[models.DjmdContent]:
        """Get contents in a hot cue banklist by ID."""
        return self._db.get_hot_cue_banklist_contents(banklist_id)

    def get_hot_cue_banklist_cues(self, banklist_id: str) -> List[models.HotCueBanklistCue]:
        """Get cues in a hot cue banklist by ID."""
        return self._db.get_hot_cue_banklist_cues(banklist_id)

    # -- Key --------------------------------------------------------------------------------------

    def get_key(self) -> List[models.DjmdKey]:
        """Get all key entries."""
        return self._db.get_key()

    def get_key_by_id(self, key_id: str) -> Optional[models.DjmdKey]:
        """Get key by ID."""
        return self._db.get_key_by_id(key_id)

    def get_key_by_name(self, key_name: str) -> Optional[models.DjmdKey]:
        """Get key by name."""
        return self._db.get_key_by_name(key_name)

    def insert_key(self, name: str) -> models.DjmdKey:
        """Insert a new key."""
        return self._db.insert_key(name)

    def update_key(self, key: models.DjmdKey) -> models.DjmdKey:
        """Update key."""
        return self._db.update_key(key)

    def delete_key(self, key_id: str) -> None:
        """Delete key."""
        self._db.delete_key(key_id)

    # -- Label ------------------------------------------------------------------------------------

    def get_label(self) -> List[models.DjmdLabel]:
        """Get all label entries."""
        return self._db.get_label()

    def get_label_by_id(self, label_id: str) -> Optional[models.DjmdLabel]:
        """Get label by ID."""
        return self._db.get_label_by_id(label_id)

    def get_label_by_name(self, label_name: str) -> Optional[models.DjmdLabel]:
        """Get label by ID."""
        return self._db.get_label_by_name(label_name)

    def insert_label(self, name: str) -> models.DjmdLabel:
        """Insert a new label."""
        return self._db.insert_label(name)

    def update_label(self, label: models.DjmdLabel) -> models.DjmdLabel:
        """Update label."""
        return self._db.update_label(label)

    def delete_label(self, label_id: str) -> None:
        """Delete label."""
        self._db.delete_label(label_id)

    # -- MenuItems --------------------------------------------------------------------------------

    def get_menu_item(self) -> List[models.DjmdMenuItems]:
        """Get all menu items."""
        return self._db.get_menu_item()

    def get_menu_item_by_id(self, item_id: str) -> Optional[models.DjmdMenuItems]:
        """Get menu item by ID."""
        return self._db.get_menu_item_by_id(item_id)

    # -- MixerParam -------------------------------------------------------------------------------

    def get_mixer_param(self) -> List[models.DjmdMixerParam]:
        """Get all mixer parameters."""
        return self._db.get_mixer_param()

    def get_mixer_param_by_id(self, param_id: str) -> Optional[models.DjmdMixerParam]:
        """Get mixer parameter by ID."""
        return self._db.get_mixer_param_by_id(param_id)

    # -- MyTag ------------------------------------------------------------------------------------

    def get_my_tag(self) -> List[models.DjmdMyTag]:
        """Get all my tags."""
        return self._db.get_my_tag()

    def get_my_tag_children(self, tag_id: str) -> List[models.DjmdMyTag]:
        """Get child tags by ID."""
        return self._db.get_my_tag_children(tag_id)

    def get_my_tag_by_id(self, tag_id: str) -> Optional[models.DjmdMyTag]:
        """Get my tag by ID."""
        return self._db.get_my_tag_by_id(tag_id)

    def get_my_tag_songs(self, tag_id: str) -> List[models.DjmdSongMyTag]:
        """Get songs in a my tag by ID."""
        return self._db.get_my_tag_songs(tag_id)

    def get_my_tag_contents(self, tag_id: str) -> List[models.DjmdContent]:
        """Get contents in a my tag by ID."""
        return self._db.get_my_tag_contents(tag_id)

    # -- Playlists --------------------------------------------------------------------------------

    def get_playlist(self) -> List[models.DjmdPlaylist]:
        """Get all playlists from the database."""
        return self._db.get_playlist()

    def get_playlist_tree(self) -> List[models.DjmdPlaylistTreeItem]:
        """Get all playlists from the database."""
        return self._db.get_playlist_tree()

    def get_playlist_children(self, playlist_id: str) -> List[models.DjmdPlaylist]:
        """Get child playlists by ID."""
        return self._db.get_playlist_children(playlist_id)

    def get_playlist_by_id(self, playlist_id: str) -> Optional[models.DjmdPlaylist]:
        """Get playlist by ID."""
        return self._db.get_playlist_by_id(playlist_id)

    def get_playlist_by_path(self, path: List[str]) -> Optional[models.DjmdPlaylist]:
        """Get a playlist by a path of names."""
        return self._db.get_playlist_by_path(path)

    def get_playlist_songs(self, playlist_id: str) -> List[models.DjmdSongPlaylist]:
        """Get songs in a playlist by ID."""
        return self._db.get_playlist_songs(playlist_id)

    def get_playlist_contents(self, playlist_id: str) -> List[models.DjmdContent]:
        """Get contents in a playlist by ID."""
        return self._db.get_playlist_contents(playlist_id)

    def get_playlist_song_by_id(self, song_id: str) -> models.DjmdSongPlaylist:
        """Get playlist song by ID."""
        return self._db.get_playlist_song_by_id(song_id)

    def insert_playlist(
        self,
        name: str,
        attribute: int,
        parent_id: str = None,
        seq: int = None,
        image_path: str = None,
        smart_list: str = None,
    ) -> models.DjmdPlaylist:
        """Insert a new playlist."""
        return self._db.insert_playlist(name, attribute, parent_id, seq, image_path, smart_list)

    def rename_playlist(self, playlist_id: str, name: str) -> models.DjmdPlaylist:
        """Rename a playlist."""
        return self._db.rename_playlist(playlist_id, name)

    def move_playlist(
        self, playlist_id: str, seq: int = None, parent_id: str = None
    ) -> models.DjmdPlaylist:
        """Move a playlist."""
        return self._db.move_playlist(playlist_id, seq, parent_id)

    def delete_playlist(self, playlist_id: str) -> None:
        """Delete playlist from the database."""
        self._db.delete_playlist(playlist_id)

    def insert_playlist_song(
        self, playlist_id: str, content_id: str, seq: int = None
    ) -> models.DjmdSongPlaylist:
        """Insert a new song into a playlist."""
        return self._db.insert_playlist_song(playlist_id, content_id, seq)

    def move_playlist_song(self, song_id: str, seq: int) -> models.DjmdSongPlaylist:
        """Move a song in a playlist."""
        return self._db.move_playlist_song(song_id, seq)

    def delete_playlist_song(self, song_id: str) -> None:
        """Delete song from playlist."""
        self._db.delete_playlist_song(song_id)

    # -- Property ---------------------------------------------------------------------------------

    def get_property(self) -> List[models.DjmdProperty]:
        """Get all properties."""
        return self._db.get_property()

    def get_property_by_id(self, property_id: str) -> Optional[models.DjmdProperty]:
        """Get property by ID."""
        return self._db.get_property_by_id(property_id)

    # -- CloudProperty ----------------------------------------------------------------------------

    def get_cloud_property(self) -> List[models.DjmdCloudProperty]:
        """Get all cloud properties."""
        return self._db.get_cloud_property()

    def get_cloud_property_by_id(self, property_id: str) -> Optional[models.DjmdCloudProperty]:
        """Get cloud property by ID."""
        return self._db.get_cloud_property_by_id(property_id)

    # -- RecommendLike ----------------------------------------------------------------------------

    def get_recommend_like(self) -> List[models.DjmdRecommendLike]:
        """Get all recommend likes."""
        return self._db.get_recommend_like()

    def get_recommend_like_by_id(self, recommend_id: str) -> Optional[models.DjmdRecommendLike]:
        """Get recommend like by ID."""
        return self._db.get_recommend_like_by_id(recommend_id)

    # -- RelatedTracks ----------------------------------------------------------------------------

    def get_related_tracks(self) -> List[models.DjmdRelatedTracks]:
        """Get all related tracks."""
        return self._db.get_related_tracks()

    def get_related_tracks_children(self, track_id: str) -> List[models.DjmdRelatedTracks]:
        """Get child related tracks by ID."""
        return self._db.get_related_tracks_children(track_id)

    def get_related_tracks_by_id(self, track_id: str) -> Optional[models.DjmdRelatedTracks]:
        """Get related tracks by ID."""
        return self._db.get_related_tracks_by_id(track_id)

    def get_related_tracks_songs(self, track_id: str) -> List[models.DjmdSongRelatedTracks]:
        """Get songs in related tracks by ID."""
        return self._db.get_related_tracks_songs(track_id)

    def get_related_tracks_contents(self, track_id: str) -> List[models.DjmdContent]:
        """Get contents in related tracks by ID."""
        return self._db.get_related_tracks_contents(track_id)

    # -- Sampler ----------------------------------------------------------------------------------

    def get_sampler(self) -> List[models.DjmdSampler]:
        """Get all samplers."""
        return self._db.get_sampler()

    def get_sampler_children(self, sampler_id: str) -> List[models.DjmdSampler]:
        """Get child samplers by ID."""
        return self._db.get_sampler_children(sampler_id)

    def get_sampler_by_id(self, sampler_id: str) -> Optional[models.DjmdSampler]:
        """Get sampler by ID."""
        return self._db.get_sampler_by_id(sampler_id)

    def get_sampler_songs(self, sampler_id: str) -> List[models.DjmdSongSampler]:
        """Get songs in a sampler by ID."""
        return self._db.get_sampler_songs(sampler_id)

    def get_sampler_contents(self, sampler_id: str) -> List[models.DjmdContent]:
        """Get contents in a sampler by ID."""
        return self._db.get_sampler_contents(sampler_id)

    # -- SongTagList ------------------------------------------------------------------------------

    def get_song_tag_list(self) -> List[models.DjmdSongTagList]:
        """Get all song tag lists."""
        return self._db.get_song_tag_list()

    def get_song_tag_list_by_id(self, tag_list_id: str) -> models.DjmdSongTagList:
        """Get song tag list by ID."""
        return self._db.get_song_tag_list_by_id(tag_list_id)

    # -- Sort -------------------------------------------------------------------------------------

    def get_sort(self) -> List[models.DjmdSort]:
        """Get all sort entries."""
        return self._db.get_sort()

    def get_sort_by_id(self, sort_id: str) -> Optional[models.DjmdSort]:
        """Get sort by ID."""
        return self._db.get_sort_by_id(sort_id)

    # -- ImageFile --------------------------------------------------------------------------------

    def get_image_file(self) -> List[models.ImageFile]:
        """Get all image files."""
        return self._db.get_image_file()

    def get_image_file_by_id(self, image_id: str) -> Optional[models.ImageFile]:
        """Get image file by ID."""
        return self._db.get_image_file_by_id(image_id)

    # -- SettingFile ------------------------------------------------------------------------------

    def get_setting_file(self) -> List[models.SettingFile]:
        """Get all setting files."""
        return self._db.get_setting_file()

    def get_setting_file_by_id(self, setting_id: str) -> Optional[models.SettingFile]:
        """Get setting file by ID."""
        return self._db.get_setting_file_by_id(setting_id)

    # -- UuidIDMap --------------------------------------------------------------------------------

    def get_uuid_id_map(self) -> List[models.UuidIDMap]:
        """Get all UUID ID maps."""
        return self._db.get_uuid_id_map()

    def get_uuid_id_map_by_id(self, map_id: str) -> Optional[models.UuidIDMap]:
        """Get UUID ID map by ID."""
        return self._db.get_uuid_id_map_by_id(map_id)



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-05-15

from pathlib import Path
from typing import List, Optional, Union

from ._rbox import PlaylistNode, PyRekordboxXml as _PyRekordboxXml, Track


class RekordboxXml:
    def __init__(self, path: Union[str, Path]) -> None:
        p = Path(path)
        if p.exists():
            self._xml = _PyRekordboxXml.load(str(path))
        else:
            self._xml = _PyRekordboxXml(str(path))

    @property
    def tracks(self) -> list[Track]:
        return self._xml.tracks

    @property
    def root_playlist(self) -> PlaylistNode:
        return self._xml.root_playlist

    def to_string(self) -> str:
        return self._xml.to_string()

    def dump_copy(self, path: Union[str, Path]) -> None:
        self._xml.dump_copy(str(path))

    def dump(self) -> None:
        self._xml.dump()

    def get_tracks(self) -> List[Track]:
        return self._xml.tracks

    def get_track(self, index: int) -> Optional[Track]:
        return self._xml.get_track(index)

    def get_track_by_id(self, track_id: str) -> Optional[Track]:
        return self._xml.get_track_by_id(track_id)

    def get_track_by_location(self, location: Union[str, Path]) -> Optional[Track]:
        loc = str(location).replace("\\", "/")
        return self._xml.get_track_by_location(loc)

    def get_track_by_key(self, key: Union[str, Path], key_type: int) -> Optional[Track]:
        key = str(key).replace("\\", "/")
        return self._xml.get_track_by_key(key, key_type)

    def add_track(self, track: Track) -> None:
        self._xml.add_track(track)

    def new_track(self, track_id: str, location: str) -> Track:
        return self._xml.new_track(track_id, location)

    def update_track(self, track: Track) -> None:
        self._xml.update_track(track)

    def remove_track(self, track_id: str) -> None:
        self._xml.remove_track(track_id)

    def clear_tracks(self) -> None:
        self._xml.clear_tracks()



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-05-10

from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import List, Optional, Union

from ._rbox import (
    BeatGrid,
    CueList,
    ExtendedBeatGrid,
    ExtendedCueList,
    PyAnlz as _PyAnlz,
    SongStructureData,
    TinyWaveformPreview,
    Waveform3BandDetail,
    Waveform3BandPreview,
    WaveformColorDetail,
    WaveformColorPreview,
    WaveformDetail,
    WaveformPreview,
)


class TagType(Enum):
    """Enum for tag types."""

    BeatGrid = "BeatGrid"
    ExtendedBeatGrid = "ExtendedBeatGrid"
    CueList = "CueList"
    ExtendedCueList = "ExtendedCueList"
    Path = "Path"
    VBR = "VBR"
    WaveformPreview = "WaveformPreview"
    TinyWaveformPreview = "TinyWaveformPreview"
    WaveformDetail = "WaveformDetail"
    WaveformColorPreview = "WaveformColorPreview"
    WaveformColorDetail = "WaveformColorDetail"
    Waveform3BandPreview = "Waveform3BandPreview"
    Waveform3BandDetail = "Waveform3BandDetail"
    SongStructure = "SongStructure"


@dataclass
class Beat:
    beat_number: int
    tempo: int
    time: int


@dataclass
class ExtBeat:
    beat_number: int


@dataclass
class Cue:
    hot_cue: int
    status: int
    order_first: int
    order_last: int
    cue_type: int
    time: int
    loop_time: int


@dataclass
class ExtendedCue:
    hot_cue: int
    cue_type: int
    time: int
    loop_time: int
    color: int
    loop_numerator: int
    loop_denominator: int
    comment: str
    hot_cue_color_index: int
    red: int
    green: int
    blue: int


class Anlz:
    def __init__(self, path: Union[str, Path]) -> None:
        path = Path(path)
        if not path.exists():
            raise FileNotFoundError(f"File not found: {path}")
        self._anlz = _PyAnlz(str(path))

    def dump_copy(self, path: Union[str, Path]) -> None:
        self._anlz.dump_copy(str(path))

    def dump(self) -> None:
        self._anlz.dump()

    def get_tags(self) -> List[str]:
        return self._anlz.get_tags()

    def contains(self, tag: TagType) -> bool:
        return self._anlz.contains(tag.value)

    def get_beat_grid(self) -> Optional[BeatGrid]:
        return self._anlz.get_beat_grid()

    def get_extended_beat_grid(self) -> Optional[ExtendedBeatGrid]:
        return self._anlz.get_extended_beat_grid()

    def get_hot_cues(self) -> Optional[CueList]:
        return self._anlz.get_hot_cues()

    def get_memory_cues(self) -> Optional[CueList]:
        return self._anlz.get_memory_cues()

    def get_extended_hot_cues(self) -> Optional[ExtendedCueList]:
        return self._anlz.get_extended_hot_cues()

    def get_extended_memory_cues(self) -> Optional[ExtendedCueList]:
        return self._anlz.get_extended_memory_cues()

    def get_path(self) -> Optional[str]:
        return self._anlz.get_path()

    def set_path(self, path: Union[str, Path]) -> None:
        self._anlz.set_path(str(path))

    def get_vbr_data(self) -> Optional[List[int]]:
        return self._anlz.get_vbr_data()

    def get_tiny_waveform_preview(self) -> Optional[TinyWaveformPreview]:
        return self._anlz.get_tiny_waveform_preview()

    def get_waveform_preview(self) -> Optional[WaveformPreview]:
        return self._anlz.get_waveform_preview()

    def get_waveform_detail(self) -> Optional[WaveformDetail]:
        return self._anlz.get_waveform_detail()

    def get_waveform_color_preview(self) -> Optional[WaveformColorPreview]:
        return self._anlz.get_waveform_color_preview()

    def get_waveform_color_detail(self) -> Optional[WaveformColorDetail]:
        return self._anlz.get_waveform_color_detail()

    def get_waveform_3band_preview(self) -> Optional[Waveform3BandPreview]:
        return self._anlz.get_waveform_3band_preview()

    def get_waveform_3band_detail(self) -> Optional[Waveform3BandDetail]:
        return self._anlz.get_waveform_3band_detail()

    def get_song_structure(self) -> Optional[SongStructureData]:
        return self._anlz.get_song_structure()



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-05-10

from pathlib import Path

import pytest

from rbox import Anlz

DEMO_DIR = Path(__file__).parent.parent.parent.parent / ".testdata" / "RBv6" / "demo" / "rekordbox"

ANLZ_ROOT = DEMO_DIR / "share" / "PIONEER" / "USBANLZ" / "e35" / "fa187-3f34-47e2-9880-2b33cb8d1304"
ANLZ_DAT = ANLZ_ROOT / "ANLZ0000.DAT"
ANLZ_EXT = ANLZ_ROOT / "ANLZ0000.EXT"
ANLZ_2EX = ANLZ_ROOT / "ANLZ0000.2EX"
ANLZ_FILES = [ANLZ_DAT, ANLZ_EXT, ANLZ_2EX]


@pytest.mark.parametrize("path", ANLZ_FILES)
def test_parse(path):
    Anlz(path)


@pytest.mark.parametrize("path", ANLZ_FILES)
def test_get_path(path):
    anlz = Anlz(path)
    _ = anlz.get_path()


@pytest.mark.parametrize("path", ANLZ_FILES)
def test_set_path(path):
    new = "/Some/Path"
    anlz = Anlz(path)
    anlz.set_path(new)



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-05-01

from datetime import datetime
from typing import Any, Dict, Iterable, List, Optional, Tuple

class Error(Exception):
    """The base class of the other exceptions in this module.

    Use this to catch all errors with one single except statement
    """

    ...

class DatabaseError(Exception):
    """Exception raised for errors that are related to the master.db database."""

    ...

class AnlzError(Exception):
    """Exception raised for errors that are related to the ANLZ files."""

    ...

class XmlError(Exception):
    """Exception raised for errors that are related to the Rekordbox XML handler."""

    ...

class SettingError(Exception):
    """Exception raised for errors that are related to the Rekordbox MySetting handler."""

    ...

# ---- MasterDb ------------------------------------------------------------------------------------

def is_rekordbox_running() -> bool:
    """Check if Rekordbox is running."""
    ...

class _RustMutableMapping:
    def __len__(self) -> int:
        """Return the number of items."""
    ...

    def __iter__(self) -> Iterable[str]:
        """Return an iterator over the keys."""
        ...

    def __getitem__(self, key: str) -> Any:
        """Get an item by key."""
        ...

    def __setitem__(self, key: str, value: Any) -> None:
        """Set an item by key."""
        ...

    def keys(self) -> Iterable[str]:
        """Return an iterator over the keys."""
        ...

    def values(self) -> Iterable[Any]:
        """Return an iterator over the values."""
        ...

    def items(self) -> Iterable[Tuple[str, Any]]:
        """Return an iterator over the keys and values."""
        ...

    def get(self, key: str, default: Any = None) -> Any:
        """Get an item by key or return the default value, if the key doesn't exist."""
        ...

    def update(self, data: Dict[str, Any]) -> None:
        """Update the fields from a dictionary."""
        ...

    def to_dict(self) -> Dict[str, Any]:
        """Convert the object to a dictionary."""
        ...

class _StatsFull:
    ID: str
    UUID: str
    rb_data_status: int
    rb_local_data_status: int
    rb_local_deleted: int
    rb_local_synced: int
    created_at: datetime
    updated_at: datetime
    usn: Optional[int]
    rb_local_usn: Optional[int]

class AgentRegistry(_RustMutableMapping):
    registry_id: str
    created_at: str
    updated_at: str
    id_1: Optional[str]
    id_2: Optional[str]
    int_1: Optional[int]
    int_2: Optional[int]
    str_1: Optional[str]
    str_2: Optional[str]
    date_1: Optional[str]
    date_2: Optional[str]
    text_1: Optional[str]
    text_2: Optional[str]

class CloudAgentRegistry(_StatsFull, _RustMutableMapping):
    int_1: Optional[int]
    int_2: Optional[int]
    str_1: Optional[str]
    str_2: Optional[str]
    date_1: Optional[str]
    date_2: Optional[str]
    text_1: Optional[str]
    text_2: Optional[str]

class ContentActiveCensor(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    ActiveCensors: Optional[str]
    rb_activecensor_count: Optional[int]

class ContentCue(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    Cues: Optional[str]
    rb_cue_count: Optional[int]

class ContentFile(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    Path: Optional[str]
    Hash: Optional[str]
    Size: Optional[int]
    rb_local_path: Optional[str]
    rb_insync_hash: Optional[str]
    rb_insync_local_usn: Optional[int]
    rb_file_hash_dirty: Optional[int]
    rb_local_file_status: Optional[int]
    rb_in_progress: Optional[int]
    rb_process_type: Optional[int]
    rb_temp_path: Optional[str]
    rb_priority: Optional[int]
    rb_file_size_dirty: Optional[int]

class DjmdActiveCensor(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    InMsec: Optional[int]
    OutMsec: Optional[int]
    Info: Optional[int]
    ParameterList: Optional[str]
    ContentUUID: Optional[str]

class DjmdAlbum(_StatsFull, _RustMutableMapping):
    Name: Optional[str]
    AlbumArtistID: Optional[str]
    ImagePath: Optional[str]
    Compilation: Optional[int]
    SearchStr: Optional[str]

class DjmdArtist(_StatsFull, _RustMutableMapping):
    Name: Optional[str]
    SearchStr: Optional[str]

class DjmdCategory(_StatsFull, _RustMutableMapping):
    MenuItemID: Optional[str]
    Seq: Optional[int]
    Disable: Optional[int]
    InfoOrder: Optional[int]

class DjmdColor(_StatsFull, _RustMutableMapping):
    ColorCode: Optional[str]
    SortKey: Optional[int]
    Commnt: Optional[str]

class DjmdContent(_StatsFull, _RustMutableMapping):
    FolderPath: Optional[str]
    FileNameL: Optional[str]
    FileNameS: Optional[str]
    Title: Optional[str]
    ArtistID: Optional[str]
    AlbumID: Optional[str]
    GenreID: Optional[str]
    BPM: Optional[int]
    Length: Optional[int]
    TrackNo: Optional[int]
    BitRate: Optional[int]
    BitDepth: Optional[int]
    Commnt: Optional[str]
    FileType: Optional[int]
    Rating: Optional[int]
    ReleaseYear: Optional[int]
    RemixerID: Optional[str]
    LabelID: Optional[str]
    OrgArtistID: Optional[str]
    KeyID: Optional[str]
    StockDate: Optional[str]
    ColorID: Optional[str]
    DJPlayCount: Optional[int]
    ImagePath: Optional[str]
    MasterDBID: Optional[str]
    MasterSongID: Optional[str]
    AnalysisDataPath: Optional[str]
    SearchStr: Optional[str]
    FileSize: Optional[int]
    DiscNo: Optional[int]
    ComposerID: Optional[str]
    Subtitle: Optional[str]
    SampleRate: Optional[int]
    DisableQuantize: Optional[int]
    Analysed: Optional[int]
    ReleaseDate: Optional[str]
    DateCreated: Optional[str]
    ContentLink: Optional[int]
    Tag: Optional[str]
    ModifiedByRBM: Optional[str]
    HotCueAutoLoad: Optional[str]
    DeliveryControl: Optional[str]
    DeliveryComment: Optional[str]
    CueUpdated: Optional[str]
    AnalysisUpdated: Optional[str]
    TrackInfoUpdated: Optional[str]
    Lyricist: Optional[str]
    ISRC: Optional[str]
    SamplerTrackInfo: Optional[int]
    SamplerPlayOffset: Optional[int]
    SamplerGain: Optional[float]
    VideoAssociate: Optional[str]
    LyricStatus: Optional[int]
    ServiceID: Optional[int]
    OrgFolderPath: Optional[str]
    Reserved1: Optional[str]
    Reserved2: Optional[str]
    Reserved3: Optional[str]
    Reserved4: Optional[str]
    ExtInfo: Optional[str]
    rb_file_id: Optional[str]
    DeviceID: Optional[str]
    rb_LocalFolderPath: Optional[str]
    SrcID: Optional[str]
    SrcTitle: Optional[str]
    SrcArtistName: Optional[str]
    SrcAlbumName: Optional[str]
    SrcLength: Optional[int]

class DjmdCue(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    InMsec: Optional[int]
    InFrame: Optional[int]
    InMpegFrame: Optional[int]
    InMpegAbs: Optional[int]
    OutMsec: Optional[int]
    OutFrame: Optional[int]
    OutMpegFrame: Optional[int]
    OutMpegAbs: Optional[int]
    Kind: Optional[int]
    Color: Optional[int]
    ColorTableIndex: Optional[int]
    ActiveLoop: Optional[int]
    Comment: Optional[str]
    BeatLoopSize: Optional[int]
    CueMicrosec: Optional[int]
    InPointSeekInfo: Optional[str]
    OutPointSeekInfo: Optional[str]
    ContentUUID: Optional[str]

class DjmdDevice(_StatsFull, _RustMutableMapping):
    MasterDBID: Optional[str]
    Name: Optional[str]

class DjmdGenre(_StatsFull, _RustMutableMapping):
    Name: Optional[str]

class DjmdHistory(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]
    DateCreated: Optional[str]

class DjmdSongHistory(_StatsFull, _RustMutableMapping):
    HistoryID: Optional[str]
    ContentID: Optional[str]
    TrackNo: Optional[int]

class DjmdHotCueBanklist(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    ImagePath: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]

class DjmdSongHotCueBanklist(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    TrackNo: Optional[int]
    CueID: Optional[str]
    InMsec: Optional[int]
    InFrame: Optional[int]
    InMpegFrame: Optional[int]
    InMpegAbs: Optional[int]
    OutMsec: Optional[int]
    OutFrame: Optional[int]
    OutMpegFrame: Optional[int]
    OutMpegAbs: Optional[int]
    Color: Optional[int]
    ColorTableIndex: Optional[int]
    ActiveLoop: Optional[int]
    Comment: Optional[str]
    BeatLoopSize: Optional[int]
    CueMicrosec: Optional[int]
    InPointSeekInfo: Optional[str]
    OutPointSeekInfo: Optional[str]
    HotCueBanklistUUID: Optional[str]

class DjmdKey(_StatsFull, _RustMutableMapping):
    ScaleName: Optional[str]
    Seq: Optional[int]

class DjmdLabel(_StatsFull, _RustMutableMapping):
    Name: Optional[str]

class DjmdMenuItems(_StatsFull, _RustMutableMapping):
    Class: Optional[int]
    Name: Optional[str]

class DjmdMixerParam(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    GainHigh: Optional[int]
    GainLow: Optional[int]
    PeakHigh: Optional[int]
    PeakLow: Optional[int]

class DjmdMyTag(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]

class DjmdSongMyTag(_StatsFull, _RustMutableMapping):
    MyTagID: Optional[str]
    ContentID: Optional[str]
    TrackNo: Optional[int]

class DjmdPlaylist(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    ImagePath: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]
    SmartList: Optional[str]

class DjmdPlaylistTreeItem(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    ImagePath: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]
    SmartList: Optional[str]

    Children: List["DjmdPlaylistTreeItem"]

class DjmdSongPlaylist(_StatsFull, _RustMutableMapping):
    PlaylistID: Optional[str]
    ContentID: Optional[str]
    TrackNo: Optional[int]

class DjmdProperty(_RustMutableMapping):
    DBID: str
    created_at: str
    updated_at: str
    DBVersion: Optional[str]
    BaseDBDrive: Optional[str]
    CurrentDBDrive: Optional[str]
    DeviceID: Optional[str]
    Reserved1: Optional[str]
    Reserved2: Optional[str]
    Reserved3: Optional[str]
    Reserved4: Optional[str]
    Reserved5: Optional[str]

class DjmdCloudProperty(_StatsFull, _RustMutableMapping):
    Reserved1: Optional[str]
    Reserved2: Optional[str]
    Reserved3: Optional[str]
    Reserved4: Optional[str]
    Reserved5: Optional[str]

class DjmdRecommendLike(_StatsFull, _RustMutableMapping):
    ContentID1: Optional[str]
    ContentID2: Optional[str]
    LikeRate: Optional[int]
    DataCreatedH: Optional[int]
    DataCreatedL: Optional[int]

class DjmdRelatedTracks(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]
    Criteria: Optional[str]

class DjmdSongRelatedTracks(_StatsFull, _RustMutableMapping):
    RelatedTracksID: Optional[str]
    ContentID: Optional[str]
    TrackNo: Optional[int]

class DjmdSampler(_StatsFull, _RustMutableMapping):
    Seq: Optional[int]
    Name: Optional[str]
    Attribute: Optional[int]
    ParentID: Optional[str]

class DjmdSongSampler(_StatsFull, _RustMutableMapping):
    SamplerID: Optional[str]
    ContentID: Optional[str]
    TrackNo: Optional[int]

class DjmdSongTagList(_StatsFull, _RustMutableMapping):
    ContentID: Optional[str]
    TrackNo: Optional[int]

class DjmdSort(_StatsFull, _RustMutableMapping):
    MenuItemID: Optional[str]
    Seq: Optional[int]
    Disable: Optional[int]

class HotCueBanklistCue(_StatsFull, _RustMutableMapping):
    HotCueBanklistID: Optional[str]
    Cues: Optional[str]
    rb_cue_count: Optional[int]

class ImageFile(_StatsFull, _RustMutableMapping):
    TableName: Optional[str]
    TargetUUID: Optional[str]
    TargetID: Optional[str]
    Path: Optional[str]
    Hash: Optional[str]
    Size: Optional[int]
    rb_local_path: Optional[str]
    rb_insync_hash: Optional[str]
    rb_insync_local_usn: Optional[int]
    rb_file_hash_dirty: Optional[int]
    rb_local_file_status: Optional[int]
    rb_in_progress: Optional[int]
    rb_process_type: Optional[int]
    rb_temp_path: Optional[str]
    rb_priority: Optional[int]
    rb_file_size_dirty: Optional[int]

class SettingFile(_StatsFull, _RustMutableMapping):
    Path: Optional[str]
    Hash: Optional[str]
    Size: Optional[int]
    rb_local_path: Optional[str]
    rb_insync_hash: Optional[str]
    rb_insync_local_usn: Optional[int]
    rb_file_hash_dirty: Optional[int]
    rb_file_size_dirty: Optional[int]

class UuidIDMap(_StatsFull, _RustMutableMapping):
    TableName: Optional[str]
    TargetUUID: Optional[str]
    CurrentID: Optional[str]

class PyMasterDb:
    def __init__(self, path: str) -> None: ...
    @classmethod
    def open(cls) -> "PyMasterDb": ...
    def set_unsafe_writes(self, bool: bool) -> None: ...
    def get_agent_registry(self) -> List[AgentRegistry]: ...
    def get_agent_registry_by_id(self, registry_id: str) -> Optional[AgentRegistry]: ...
    def get_local_usn(self) -> int: ...
    def get_cloud_agent_registry(self) -> List[CloudAgentRegistry]: ...
    def get_cloud_agent_registry_by_id(self, registry_id: str) -> Optional[CloudAgentRegistry]: ...
    def get_content_active_censor(self) -> List[ContentActiveCensor]: ...
    def get_content_active_censor_by_id(self, censor_id: str) -> Optional[ContentActiveCensor]: ...
    def get_content_cue(self) -> List[ContentCue]: ...
    def get_content_cue_by_id(self, cue_id: str) -> Optional[ContentCue]: ...
    def get_content_file(self) -> List[ContentFile]: ...
    def get_content_file_by_id(self, file_id: str) -> Optional[ContentFile]: ...
    def get_active_censor(self) -> List[DjmdActiveCensor]: ...
    def get_active_censor_by_id(self, censor_id: str) -> Optional[DjmdActiveCensor]: ...
    def get_album(self) -> List[DjmdAlbum]: ...
    def get_album_by_id(self, album_id: str) -> Optional[DjmdAlbum]: ...
    def get_album_by_name(self, album_name: str) -> Optional[DjmdAlbum]: ...
    def insert_album(
        self, name: str, artist_id: str = None, image_path: str = None, compilation: int = None
    ) -> DjmdAlbum: ...
    def update_album(self, album: DjmdAlbum) -> DjmdAlbum: ...
    def delete_album(self, album_id: str) -> None: ...
    def get_artist(self) -> List[DjmdArtist]: ...
    def get_artist_by_id(self, artist_id: str) -> Optional[DjmdArtist]: ...
    def get_artist_by_name(self, artist_name: str) -> Optional[DjmdArtist]: ...
    def insert_artist(self, name: str) -> DjmdArtist: ...
    def update_artist(self, artist: DjmdArtist) -> DjmdArtist: ...
    def delete_artist(self, artist_id: str) -> None: ...
    def get_category(self) -> List[DjmdCategory]: ...
    def get_category_by_id(self, category_id: str) -> Optional[DjmdCategory]: ...
    def get_color(self) -> List[DjmdColor]: ...
    def get_color_by_id(self, color_id: str) -> Optional[DjmdColor]: ...
    def get_content(self) -> List[DjmdContent]: ...
    def get_content_by_id(self, content_id: str) -> Optional[DjmdContent]: ...
    def get_content_by_path(self, path: str) -> Optional[DjmdContent]: ...
    def get_content_anlz_dir(self, content_id: str) -> str: ...
    def get_content_anlz_paths(self, content_id: str) -> Dict[str, str]: ...
    def insert_content(self, path: str) -> DjmdContent: ...
    def update_content(self, content: DjmdContent) -> None: ...
    def update_content_album(self, content_id: str, name: str) -> None: ...
    def update_content_artist(self, content_id: str, name: str) -> None: ...
    def update_content_remixer(self, content_id: str, name: str) -> None: ...
    def update_content_original_artist(self, content_id: str, name: str) -> None: ...
    def update_content_composer(self, content_id: str, name: str) -> None: ...
    def update_content_genre(self, content_id: str, name: str) -> None: ...
    def update_content_label(self, content_id: str, name: str) -> None: ...
    def update_content_key(self, content_id: str, name: str) -> None: ...
    # def delete_content(self, content_id: str) -> None: ...
    def get_cue(self) -> List[DjmdCue]: ...
    def get_cue_by_id(self, cue_id: str) -> Optional[DjmdCue]: ...
    def get_device(self) -> List[DjmdDevice]: ...
    def get_device_by_id(self, device_id: str) -> Optional[DjmdDevice]: ...
    def get_genre(self) -> List[DjmdGenre]: ...
    def get_genre_by_id(self, genre_id: str) -> Optional[DjmdGenre]: ...
    def get_genre_by_name(self, genre_name: str) -> Optional[DjmdGenre]: ...
    def insert_genre(self, name: str) -> DjmdGenre: ...
    def update_genre(self, genre: DjmdGenre) -> DjmdGenre: ...
    def delete_genre(self, genre_id: str) -> None: ...
    def get_history(self) -> List[DjmdHistory]: ...
    def get_history_by_id(self, history_id: str) -> Optional[DjmdHistory]: ...
    def get_history_songs(self, history_id: str) -> List[DjmdSongHistory]: ...
    def get_history_contents(self, history_id: str) -> List[DjmdContent]: ...
    def get_hot_cue_banklist(self) -> List[DjmdHotCueBanklist]: ...
    def get_hot_cue_banklist_by_id(self, banklist_id: str) -> Optional[DjmdHotCueBanklist]: ...
    def get_hot_cue_banklist_children(self, banklist_id: str) -> List[DjmdHotCueBanklist]: ...
    def get_hot_cue_banklist_songs(self, banklist_id: str) -> List[DjmdSongHotCueBanklist]: ...
    def get_hot_cue_banklist_contents(self, banklist_id: str) -> List[DjmdContent]: ...
    def get_hot_cue_banklist_cues(self, banklist_id: str) -> List[HotCueBanklistCue]: ...
    def get_key(self) -> List[DjmdKey]: ...
    def get_key_by_id(self, key_id: str) -> Optional[DjmdKey]: ...
    def get_key_by_name(self, key_name: str) -> Optional[DjmdKey]: ...
    def insert_key(self, name: str) -> DjmdKey: ...
    def update_key(self, key: DjmdKey) -> DjmdKey: ...
    def delete_key(self, key_id: str) -> None: ...
    def get_label(self) -> List[DjmdLabel]: ...
    def get_label_by_id(self, label_id: str) -> Optional[DjmdLabel]: ...
    def get_label_by_name(self, label_name: str) -> Optional[DjmdLabel]: ...
    def insert_label(self, name: str) -> DjmdLabel: ...
    def update_label(self, label: DjmdLabel) -> DjmdLabel: ...
    def delete_label(self, label_id: str) -> None: ...
    def get_menu_item(self) -> List[DjmdMenuItems]: ...
    def get_menu_item_by_id(self, item_id: str) -> Optional[DjmdMenuItems]: ...
    def get_mixer_param(self) -> List[DjmdMixerParam]: ...
    def get_mixer_param_by_id(self, param_id: str) -> Optional[DjmdMixerParam]: ...
    def get_my_tag(self) -> List[DjmdMyTag]: ...
    def get_my_tag_children(self, tag_id: str) -> List[DjmdMyTag]: ...
    def get_my_tag_by_id(self, tag_id: str) -> Optional[DjmdMyTag]: ...
    def get_my_tag_songs(self, tag_id: str) -> List[DjmdSongMyTag]: ...
    def get_my_tag_contents(self, tag_id: str) -> List[DjmdContent]: ...
    def get_playlist(self) -> List[DjmdPlaylist]: ...
    def get_playlist_tree(self) -> List[DjmdPlaylistTreeItem]: ...
    def get_playlist_children(self, playlist_id: str) -> List[DjmdPlaylist]: ...
    def get_playlist_by_id(self, playlist_id: str) -> Optional[DjmdPlaylist]: ...
    def get_playlist_by_path(self, path: List[str]) -> Optional[DjmdPlaylist]: ...
    def get_playlist_songs(self, playlist_id: str) -> List[DjmdSongPlaylist]: ...
    def get_playlist_contents(self, playlist_id: str) -> List[DjmdContent]: ...
    def get_playlist_song_by_id(self, song_id: str) -> Optional[DjmdSongPlaylist]: ...
    def insert_playlist(
        self,
        name: str,
        attribute: int,
        parent_id: str = None,
        seq: int = None,
        image_path: str = None,
        smart_list: str = None,
    ) -> DjmdPlaylist: ...
    def rename_playlist(self, playlist_id: str, name: str) -> DjmdPlaylist: ...
    def move_playlist(
        self, playlist_id: str, seq: int = None, parent_id: str = None
    ) -> DjmdPlaylist: ...
    def delete_playlist(self, playlist_id: str) -> None: ...
    def insert_playlist_song(
        self, playlist_id: str, content_id: str, seq: int = None
    ) -> DjmdSongPlaylist: ...
    def move_playlist_song(self, song_id: str, seq: int) -> DjmdSongPlaylist: ...
    def delete_playlist_song(self, song_id: str) -> None: ...
    def get_property(self) -> List[DjmdProperty]: ...
    def get_property_by_id(self, property_id: str) -> Optional[DjmdProperty]: ...
    def get_cloud_property(self) -> List[DjmdCloudProperty]: ...
    def get_cloud_property_by_id(self, property_id: str) -> Optional[DjmdCloudProperty]: ...
    def get_recommend_like(self) -> List[DjmdRecommendLike]: ...
    def get_recommend_like_by_id(self, recommend_id: str) -> Optional[DjmdRecommendLike]: ...
    def get_related_tracks(self) -> List[DjmdRelatedTracks]: ...
    def get_related_tracks_children(self, track_id: str) -> List[DjmdRelatedTracks]: ...
    def get_related_tracks_by_id(self, track_id: str) -> Optional[DjmdRelatedTracks]: ...
    def get_related_tracks_songs(self, track_id: str) -> List[DjmdSongRelatedTracks]: ...
    def get_related_tracks_contents(self, track_id: str) -> List[DjmdContent]: ...
    def get_sampler(self) -> List[DjmdSampler]: ...
    def get_sampler_children(self, sampler_id: str) -> List[DjmdSampler]: ...
    def get_sampler_by_id(self, sampler_id: str) -> Optional[DjmdSampler]: ...
    def get_sampler_songs(self, sampler_id: str) -> List[DjmdSongSampler]: ...
    def get_sampler_contents(self, sampler_id: str) -> List[DjmdContent]: ...
    def get_song_tag_list(self) -> List[DjmdSongTagList]: ...
    def get_song_tag_list_by_id(self, tag_list_id: str) -> Optional[DjmdSongTagList]: ...
    def get_sort(self) -> List[DjmdSort]: ...
    def get_sort_by_id(self, sort_id: str) -> Optional[DjmdSort]: ...
    def get_image_file(self) -> List[ImageFile]: ...
    def get_image_file_by_id(self, image_id: str) -> Optional[ImageFile]: ...
    def get_setting_file(self) -> List[SettingFile]: ...
    def get_setting_file_by_id(self, setting_id: str) -> Optional[SettingFile]: ...
    def get_uuid_id_map(self) -> List[UuidIDMap]: ...
    def get_uuid_id_map_by_id(self, uuid_id: str) -> Optional[UuidIDMap]: ...

# ---- ANLZ ----------------------------------------------------------------------------------------

class Beat:
    beat_number: int
    tempo: int
    time: int

class ExtBeat:
    beat_number: int

class BeatGrid:
    beats: List[Beat]

class ExtendedBeatGrid:
    bpm: List[Beat]
    beats: List[ExtBeat]

class Cue:
    hot_cue: int
    status: int
    order_first: int
    order_last: int
    cue_type: int
    time: int
    loop_time: int

class ExtendedCue:
    hot_cue: int
    cue_type: int
    time: int
    loop_time: int
    color: int
    loop_numerator: int
    loop_denominator: int
    comment: str
    hot_cue_color_index: int
    hot_cue_color_red: int
    hot_cue_color_green: int
    hot_cue_color_blue: int

class CueList:
    list_type: int
    cues: List[Cue]

class ExtendedCueList:
    list_type: int
    cues: List[ExtendedCue]

class TinyWaveformColumn:
    height: int

class TinyWaveformPreview:
    data: List[TinyWaveformColumn]

class WaveformColumn:
    height: int
    whiteness: int

class WaveformPreview:
    data: List[WaveformColumn]

class WaveformDetail:
    data: List[WaveformColumn]

class WaveformColorPreviewColumn:
    energy_bottom_half_freq: int
    energy_bottom_third_freq: int
    energy_mid_third_freq: int
    energy_top_third_freq: int

class WaveformColorPreview:
    data: List[WaveformColorPreviewColumn]

class WaveformColorDetailColumn:
    red: int
    green: int
    blue: int
    height: int

class WaveformColorDetail:
    data: List[WaveformColorDetailColumn]

class Waveform3BandColumn:
    mid: int
    high: int
    low: int

class Waveform3BandPreview:
    data: List[Waveform3BandColumn]

class Waveform3BandDetail:
    data: List[Waveform3BandColumn]

class Phrase:
    index: int
    beat: int
    kind: int
    k1: int
    k2: int
    b: int
    beat2: int
    beat3: int
    beat4: int
    k3: int
    fill: int
    beat_fill: int

class SongStructureData:
    mood: int
    end_beat: int
    bank: int
    phrases: List[Phrase]

class PyAnlz:
    def __init__(self, path: str) -> None: ...
    def dump_copy(self, path: str) -> None: ...
    def dump(self) -> None: ...
    def get_tags(self) -> List[str]: ...
    def contains(self, tag_type: str) -> bool: ...
    def get_beat_grid(self) -> Optional[BeatGrid]: ...
    def get_extended_beat_grid(self) -> Optional[ExtendedBeatGrid]: ...
    def get_hot_cues(self) -> Optional[CueList]: ...
    def get_memory_cues(self) -> Optional[CueList]: ...
    def get_extended_hot_cues(self) -> Optional[ExtendedCueList]: ...
    def get_extended_memory_cues(self) -> Optional[ExtendedCueList]: ...
    def get_path(self) -> Optional[str]: ...
    def set_path(self, path: str) -> None: ...
    def get_vbr_data(self) -> Optional[List[int]]: ...
    def get_tiny_waveform_preview(self) -> Optional[TinyWaveformPreview]: ...
    def get_waveform_preview(self) -> Optional[WaveformPreview]: ...
    def get_waveform_detail(self) -> Optional[WaveformDetail]: ...
    def get_waveform_color_preview(self) -> Optional[WaveformColorPreview]: ...
    def get_waveform_color_detail(self) -> Optional[WaveformColorDetail]: ...
    def get_waveform_3band_preview(self) -> Optional[Waveform3BandPreview]: ...
    def get_waveform_3band_detail(self) -> Optional[Waveform3BandDetail]: ...
    def get_song_structure(self) -> Optional[SongStructureData]: ...

# ---- XML -----------------------------------------------------------------------------------------

class Tempo(_RustMutableMapping):
    inizio: float
    bpm: float
    metro: str
    battito: int

    def __init__(self, inizio: float, bpm: float, metro: str, battito: int) -> None: ...

class PositionMark(_RustMutableMapping):
    name: str
    mark_type: int
    num: int
    start: float
    end: Optional[float]

    def __init__(
        self, name: str, mark_type: int, num: int, start: float, end: Optional[float]
    ) -> None: ...

class PlaylistTrack:
    key: str

class Track(_RustMutableMapping):
    trackid: str
    location: str
    tempos: List[Tempo]
    position_marks: List[PositionMark]
    name: Optional[str] = None
    artist: Optional[str] = None
    composer: Optional[str] = None
    album: Optional[str] = None
    grouping: Optional[str] = None
    genre: Optional[str] = None
    kind: Optional[str] = None
    size: Optional[int] = None
    totaltime: Optional[float] = None
    discnumber: Optional[int] = None
    tracknumber: Optional[int] = None
    year: Optional[int] = None
    averagebpm: Optional[float] = None
    datemodified: Optional[datetime] = None
    dateadded: Optional[datetime] = None
    bitrate: Optional[int] = None
    samplerate: Optional[float] = None
    comments: Optional[str] = None
    playcount: Optional[int] = None
    lastplayed: Optional[datetime] = None
    rating: Optional[int] = None
    remixer: Optional[str] = None
    tonality: Optional[str] = None
    label: Optional[str] = None
    mix: Optional[str] = None
    color: Optional[str] = None

    def __init__(self, track_id: str, location: str) -> None: ...

class PlaylistNode:
    name: str
    node_type: int
    key_type: Optional[int]
    tracks: Optional[List[PlaylistTrack]]
    nodes: Optional[List[PlaylistNode]]
    node_path: List[str]

    def __init__(self, node_type: int) -> None: ...
    @classmethod
    def playlist(cls, name: str, key_type: int) -> "PlaylistNode": ...
    @classmethod
    def folder(cls, name: str) -> "PlaylistNode": ...
    def add_node(self, node: "PlaylistNode") -> None: ...
    def new_playlist(self, name: str, key_type: int) -> "PlaylistNode": ...
    def new_folder(self, name: str) -> "PlaylistNode": ...
    def remove_node(self, index: int) -> None: ...
    def clear_nodes(self) -> None: ...
    def get_child(self, index: int) -> "PlaylistNode": ...
    def find_child(self, name: str) -> Optional["PlaylistNode"]: ...
    def get_track(self, index: int) -> PlaylistTrack: ...
    def new_track(self, track_key: str) -> None: ...
    def add_track(self, track: PlaylistTrack) -> None: ...
    def remove_track(self, track_key: str) -> None: ...
    def clear_tracks(self) -> None: ...

class PyRekordboxXml:
    root_playlist: PlaylistNode
    tracks: List[Track]

    def __init__(self, path: str) -> None: ...
    @classmethod
    def load(cls, path: str) -> "PyRekordboxXml": ...
    def to_string(self) -> str: ...
    def dump_copy(self, path: str) -> None: ...
    def dump(self) -> None: ...
    # def get_tracks(self) -> List[Track]: ...
    def get_track(self, index: int) -> Optional[Track]: ...
    def get_track_by_id(self, track_id: str) -> Optional[Track]: ...
    def get_track_by_location(self, location: str) -> Optional[Track]: ...
    def get_track_by_key(self, key: str, key_type: int) -> Optional[Track]: ...
    def add_track(self, track: Track) -> None: ...
    def new_track(self, track_id: str, location: str) -> Track: ...
    def update_track(self, track: Track) -> None: ...
    def remove_track(self, track_id: str) -> None: ...
    def clear_tracks(self) -> None: ...
    def get_playlist_root(self) -> PlaylistNode: ...

# ---- Settings ------------------------------------------------------------------------------------

class PySetting:
    def __init__(self, path: str) -> None: ...
    @classmethod
    def new_mysetting(cls, path: str) -> "PySetting": ...
    @classmethod
    def new_mysetting2(cls, path: str) -> "PySetting": ...
    @classmethod
    def new_djmmysetting(cls, path: str) -> "PySetting": ...
    @classmethod
    def new_devsetting(cls, path: str) -> "PySetting": ...
    @classmethod
    def load(cls, path: str) -> "PySetting": ...
    def dump_copy(self, path: str) -> None: ...
    def dump(self) -> None: ...
    def get_on_air_display(self) -> str: ...
    def set_on_air_display(self, on_air_display: str) -> None: ...
    def get_lcd_brightness(self) -> str: ...
    def set_lcd_brightness(self, lcd_brightness: str) -> None: ...
    def get_quantize(self) -> str: ...
    def set_quantize(self, quantize: str) -> None: ...
    def get_auto_cue_level(self) -> str: ...
    def set_auto_cue_level(self, auto_cue_level: str) -> None: ...
    def get_language(self) -> str: ...
    def set_language(self, language: str) -> None: ...
    def get_jog_ring_brightness(self) -> str: ...
    def set_jog_ring_brightness(self, jog_ring_brightness: str) -> None: ...
    def get_jog_ring_indicator(self) -> str: ...
    def set_jog_ring_indicator(self, jog_ring_indicator: str) -> None: ...
    def get_slip_flashing(self) -> str: ...
    def set_slip_flashing(self, slip_flashing: str) -> None: ...
    def get_disc_slot_illumination(self) -> str: ...
    def set_disc_slot_illumination(self, disc_slot_illumination: str) -> None: ...
    def get_eject_lock(self) -> str: ...
    def set_eject_lock(self, eject_lock: str) -> None: ...
    def get_sync(self) -> str: ...
    def set_sync(self, sync: str) -> None: ...
    def get_play_mode(self) -> str: ...
    def set_play_mode(self, play_mode: str) -> None: ...
    def get_quantize_beat_value(self) -> str: ...
    def set_quantize_beat_value(self, quantize_beat_value: str) -> None: ...
    def get_hotcue_autoload(self) -> str: ...
    def set_hotcue_autoload(self, hotcue_autoload: str) -> None: ...
    def get_hotcue_color(self) -> str: ...
    def set_hotcue_color(self, hotcue_color: str) -> None: ...
    def get_needle_lock(self) -> str: ...
    def set_needle_lock(self, needle_lock: str) -> None: ...
    def get_time_mode(self) -> str: ...
    def set_time_mode(self, time_mode: str) -> None: ...
    def get_jog_mode(self) -> str: ...
    def set_jog_mode(self, jog_mode: str) -> None: ...
    def get_auto_cue(self) -> str: ...
    def set_auto_cue(self, auto_cue: str) -> None: ...
    def get_master_tempo(self) -> str: ...
    def set_master_tempo(self, master_tempo: str) -> None: ...
    def get_tempo_range(self) -> str: ...
    def set_tempo_range(self, tempo_range: str) -> None: ...
    def get_phase_meter(self) -> str: ...
    def set_phase_meter(self, phase_meter: str) -> None: ...
    def get_vinyl_speed_adjust(self) -> str: ...
    def set_vinyl_speed_adjust(self, vinyl_speed_adjust: str) -> None: ...
    def get_jog_display_mode(self) -> str: ...
    def set_jog_display_mode(self, jog_display_mode: str) -> None: ...
    def get_pad_button_brightness(self) -> str: ...
    def set_pad_button_brightness(self, pad_button_brightness: str) -> None: ...
    def get_jog_lcd_brightness(self) -> str: ...
    def set_jog_lcd_brightness(self, jog_lcd_brightness: str) -> None: ...
    def get_waveform_divisions(self) -> str: ...
    def set_waveform_divisions(self, waveform_divisions: str) -> None: ...
    def get_waveform(self) -> str: ...
    def set_waveform(self, waveform: str) -> None: ...
    def get_beat_jump_beat_value(self) -> str: ...
    def set_beat_jump_beat_value(self, beat_jump_beat_value: str) -> None: ...
    def get_channel_fader_curve(self) -> str: ...
    def set_channel_fader_curve(self, channel_fader_curve: str) -> None: ...
    def get_crossfader_curve(self) -> str: ...
    def set_crossfader_curve(self, crossfader_curve: str) -> None: ...
    def get_headphones_pre_eq(self) -> str: ...
    def set_headphones_pre_eq(self, headphones_pre_eq: str) -> None: ...
    def get_headphones_mono_split(self) -> str: ...
    def set_headphones_mono_split(self, headphones_mono_split: str) -> None: ...
    def get_beat_fx_quantize(self) -> str: ...
    def set_beat_fx_quantize(self, beat_fx_quantize: str) -> None: ...
    def get_mic_low_cut(self) -> str: ...
    def set_mic_low_cut(self, mic_low_cut: str) -> None: ...
    def get_talk_over_mode(self) -> str: ...
    def set_talk_over_mode(self, talk_over_mode: str) -> None: ...
    def get_talk_over_level(self) -> str: ...
    def set_talk_over_level(self, talk_over_level: str) -> None: ...
    def get_midi_channel(self) -> str: ...
    def set_midi_channel(self, midi_channel: str) -> None: ...
    def get_midi_button_type(self) -> str: ...
    def set_midi_button_type(self, midi_button_type: str) -> None: ...
    def get_mixer_display_brightness(self) -> str: ...
    def set_mixer_display_brightness(self, mixer_display_brightness: str) -> None: ...
    def get_mixer_indicator_brightness(self) -> str: ...
    def set_mixer_indicator_brightness(self, mixer_indicator_brightness: str) -> None: ...
    def get_channel_fader_curve_long_fader(self) -> str: ...
    def set_channel_fader_curve_long_fader(self, channel_fader_curve_long_fader: str) -> None: ...
    def get_overview_waveform_type(self) -> str: ...
    def set_overview_waveform_type(self, overview_waveform_type: str) -> None: ...
    def get_waveform_color(self) -> str: ...
    def set_waveform_color(self, waveform_color: str) -> None: ...
    def get_key_display_format(self) -> str: ...
    def set_key_display_format(self, key_display_format: str) -> None: ...
    def get_waveform_current_position(self) -> str: ...
    def set_waveform_current_position(self, waveform_current_position: str) -> None: ...



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-05-07

import shutil
from pathlib import Path

import pytest

from rbox import MasterDb
from rbox.masterdb import models

DEMO_DIR = Path(__file__).parent.parent.parent.parent / ".testdata" / "RBv6" / "demo" / "rekordbox"
DEMO_DB_SRC = DEMO_DIR / "master.db"
DEMO_DB = DEMO_DIR / "master-copy.db"


@pytest.fixture
def db():
    """Return a clean Rekordbox database instance."""
    shutil.copy(DEMO_DB_SRC, DEMO_DB)
    db = MasterDb(DEMO_DB)
    yield db


def test_open_master_db(db):
    pass


def test_get_local_usn(db):
    usn = db.get_local_usn()
    assert usn > 0


# -- Album ----------------------------------------------------------------------------------------


def test_get_albums(db):
    _ = db.get_album()


def test_get_album(db):
    item = db.get_album_by_id("1234")
    assert item is None


def test_insert_album(db):
    item = db.insert_album("Name")
    assert isinstance(item, models.DjmdAlbum)

    id_ = item.ID
    album = db.get_album_by_id(id_)
    assert album is not None


def test_get_album_by_name(db):
    item = db.get_album_by_name("Name")
    assert item is None

    db.insert_album("Name")

    item = db.get_album_by_name("Name")
    assert item is not None


def test_update_album(db):
    item = db.insert_album("Name")
    assert isinstance(item, models.DjmdAlbum)

    id_ = item.ID
    new_item = db.get_album_by_id(id_)
    assert new_item is not None

    # Update the album name
    new_name = "New Name"
    new_item.Name = new_name
    db.update_album(new_item)

    # Verify the update
    updated = db.get_album_by_id(id_)
    assert updated is not None
    assert updated.Name == new_name



# -*- coding: utf-8 -*-
# Author: Dylan Jones
# Date:   2025-05-01

from ._rbox import (
    AnlzError,
    DatabaseError,
    Error,
    SettingError,
    XmlError,
    is_rekordbox_running,
)
from .anlz import Anlz, TagType
from .masterdb import MasterDb
from .rbxml import RekordboxXml
from .settings import Setting
